import { createSelector } from '@reduxjs/toolkit';
import { rootSelector } from '@/redux/root/root.selectors';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { currentSearchedBioEntityId } from '@/redux/drawer/drawer.selectors';
import { ModelElement } from '@/types/models';
import { MODEL_ELEMENTS_DEFAULT_COMPARTMENT_NAME } from '@/redux/modelElements/modelElements.constants';
import { COMPARTMENT_SBO_TERM } from '@/components/Map/MapViewer/MapViewer.constants';

export const modelElementsSelector = createSelector(rootSelector, state => state.modelElements);

export const modelElementsStateForCurrentModelSelector = createSelector(
  modelElementsSelector,
  currentModelIdSelector,
  (state, currentModelId) => state[currentModelId],
);

export const modelElementsByModelIdSelector = createSelector(
  [modelElementsSelector, (_state, modelId: number): number => modelId],
  (state, modelId) => state[modelId]?.data || [],
);

export const modelElementsCurrentModelLoadingSelector = createSelector(
  modelElementsStateForCurrentModelSelector,
  state => state?.loading,
);

export const modelElementsAnyModelLoadingSelector = createSelector(modelElementsSelector, state =>
  Object.values(state).some(modelElementState => modelElementState.loading === 'pending'),
);

export const modelElementsForCurrentModelSelector = createSelector(
  modelElementsStateForCurrentModelSelector,
  state => state?.data || [],
);

export const modelElementsWithSubmapConnectionForCurrentModelSelector = createSelector(
  modelElementsForCurrentModelSelector,
  modelElements => modelElements.filter(modelElement => Boolean(modelElement.submodel)) || [],
);

export const compartmentNameByIdSelector = createSelector(
  [
    modelElementsForCurrentModelSelector,
    (_state, compartmentId: number | undefined | null): number | undefined | null => compartmentId,
  ],
  (modelElements, compartmentId) => {
    if (!compartmentId) {
      return MODEL_ELEMENTS_DEFAULT_COMPARTMENT_NAME;
    }
    return (
      modelElements.find(modelElement => {
        return modelElement.sboTerm === COMPARTMENT_SBO_TERM && modelElement.id === compartmentId;
      })?.name || MODEL_ELEMENTS_DEFAULT_COMPARTMENT_NAME
    );
  },
);

export const currentDrawerModelElementSelector = createSelector(
  modelElementsForCurrentModelSelector,
  currentSearchedBioEntityId,
  (modelElements, currentBioEntityId): ModelElement | undefined => {
    return modelElements.find(modelElement => modelElement.id === currentBioEntityId);
  },
);

export const compartmentPathwaysSelector = createSelector(
  modelElementsSelector,
  (state): ModelElement[] => {
    const pathways: ModelElement[] = [];
    Object.values(state).forEach(modelState => {
      pathways.push(
        ...(modelState.data || []).filter(
          modelElement => modelElement.sboTerm === COMPARTMENT_SBO_TERM,
        ),
      );
    });
    return pathways;
  },
);
